#ifndef LIB_MQLPLUS_BASE_OBJECT_BINARY_TREE_NODE_MQH_INCLUDED
#define LIB_MQLPLUS_BASE_OBJECT_BINARY_TREE_NODE_MQH_INCLUDED
#property version "1.0";
/**********************************************************************************
 * Copyright (C) 2010-2022 Dominik Egert <info@freie-netze.de>
 *
 * This file is the objects include file.
 *
 * MQLplus, including this file may not be copied and/or distributed
 * without explecit permit by the author.
 * Author Dominik Egert / Freie Netze UG.
 **********************************************************************************
 *
 *  Version: 1.0
 *  State: production
 *
 *  File information
 *  ================
 *
 *
 *
*/
#ifdef DBG_MSG_TRACE_FILE_LOADER
    DBG_MSG_TRACE_FILE_LOADER;

#endif



// Incluse base node object
#include "lib_base_obj_node_tree.mqh"





/*********************************************************************************************************************************************************/
/*                                                                                                                                                       */
/* MQLplus data structures                                                                                                                               */
/*                                                                                                                                                       */
/*********************************************************************************************************************************************************/


    ///////////////////////////////////////
    //
    //  Object Tree Node
    //  (Binary in-order-algorithm)
    //

    // Base Tree object

        #define THIS_TYPE       _object_binary_t_node<USER_DATA>
        #define NODE_SUBTYPE    _object_t_node<USER_DATA, THIS_TYPE>

        template <typename USER_DATA>
        class _object_binary_t_node : public NODE_SUBTYPE
        {
            protected:
            // Local storage

                THIS_TYPE*  _parent;						// Parent node


            public:
            // Constructors / Destructor

                // Default constructor
                _object_binary_t_node() :
                    NODE_SUBTYPE(),
					_parent		(NULL)
                { };

                // Copy constructor
                _object_binary_t_node(const THIS_TYPE& p_in)
                {
                    remove(_parent);
                    NODE_SUBTYPE(p_in);
					_parent	= p_in._parent;
                };

                // Destructor
                ~_object_binary_t_node()
                { remove(_parent); };


            // Element selector functions

                // First element
                THIS_TYPE* top()
                { return(root()); }

                THIS_TYPE* root()
                {
                    THIS_TYPE* ptr_obj = ::GetPointer(this);
                    while( (ptr_obj._parent != NULL)
                        && (!_StopFlag) )
                    { ptr_obj = ptr_obj._parent; }

                    // Return
                    return(ptr_obj);
                }


            // Tree element functions

                // Insert node to tree
                THIS_TYPE* insert(THIS_TYPE* _root, const bool unique = false)
                { 
                    NODE_SUBTYPE* tmp_ptr           = _parent;
                    _root                           = NODE_SUBTYPE::insert(_root, tmp_ptr, unique); 
                    _parent                         = tmp_ptr;
                    return(_root);
                };

                // Remove node from tree
                THIS_TYPE* remove(THIS_TYPE* _root, const bool do_not_delete = false)
                { 
                    // Validate node
                    if(_root == NULL)
                    { return(NULL); }

                    // Remove
                    NODE_SUBTYPE*   sub_parent      = _parent;
                    THIS_TYPE*      tmp_ptr         = NODE_SUBTYPE::remove(sub_parent);
                    _root                           = (tmp_ptr != NULL) ? tmp_ptr : _root;
                    tmp_ptr                         = ::GetPointer(this);

                    // Check if last node
                    if( (_root == tmp_ptr)
                       && (_right == NULL)
                       && (_left == NULL) )
                    { return(NULL); }

                    // Update parents
                    // (_left holds successor node)
                    // (_right holds successors right child node)
                    // (_sub_parent holds successors right child's new parent)
                    // (_parent holds parent of node to be deleted)
                    //
                    // if leaf node, _left and _right == NULL
                    // if not leaf node _left != NULL
                    // if one child node, _left == _sub_parent and _right == _left._right
                    // if two nodes
                    //  - _left holds replacement node, 
                    //  - _right holds former replacement nodes _right
                    //  - _sub_parent holds "former replacement nodes _right" new parent

                    if(_left != NULL)
                    { 
                        // Update replacement nodes parents
                        _left._parent               = _parent; 
                    
                        // Childs parents
                        if(_left._left != NULL)
                        { _left._left._parent       = _left; }
                        
                        if(_left._right != NULL)
                        { _left._right._parent      = _left; }

                        // Subnodes childrens parent
                        if( (sub_parent != _left)
                         && (sub_parent != tmp_ptr) )
                        { 
                            if(sub_parent._left != NULL)
                            { sub_parent._left._parent  = sub_parent; }

                            if(sub_parent._right != NULL)
                            { sub_parent._right._parent = sub_parent; }
                        }
                    }
/*
                    // (sub_parent holds subtree-parent)
                    if( (sub_parent != NULL)
                     && (sub_parent != _left) )
                    {
                        // Subnodes childs parent
                        if(sub_parent._left != NULL)
                        { sub_parent._left._parent  = sub_parent; }

                        if(sub_parent._right != NULL)
                        { sub_parent._right._parent = sub_parent; }
                    }

                    // For leaf nodes, preserve parent
                    else if( (_parent != NULL) 
                          && (_right != NULL) )
                    { sub_parent                    = _parent; }
*/
                    // Update root
                    _root._parent                   = NULL;

                    // Clear node
                    _left                           = (do_not_delete) ? _left       : NULL;
                    _right                          = (do_not_delete) ? _right      : NULL;
                    _parent                         = (do_not_delete) ? sub_parent  : NULL;

                    //Return
                    return(_root);
                }


            // Operators

                // Assignment operator
                void operator=(const THIS_TYPE& p_in)
                { 
                    remove(_parent);
                    NODE_SUBTYPE::operator=((NODE_SUBTYPE)p_in);
					_parent	= p_in._parent;
                };

                void operator=(THIS_TYPE* p_in)
                {
                    remove(_parent);
                    NODE_SUBTYPE::operator=((NODE_SUBTYPE*)p_in);
					_parent	= p_in._parent;
                };

                // Access operator
                THIS_TYPE* operator[](USER_DATA& p_in)
                {
                    // Local init
                    THIS_TYPE* x = ::GetPointer(this);

                    // Loop tree
                    while( (x != NULL)
                        && (x.this_obj != p_in) )
                    { x = (x.this_obj > p_in) ? x._left : x._right; }

                    // Return
                    return(x);
                };


            public:
            // Debugging helper

                const string _node_to_string(int& count)
                { 
                    if( ( _parent != NULL) 
                     && (CheckPointer(_parent) == POINTER_INVALID) )
                    { return(NULL); }
                    count++; return(StringFormat("Parent Value: %s; Value: %s", (_parent != NULL) ? __obj_to_string(_parent.this_obj) : "Root-Node", __obj_to_string(this_obj))); };
        };

        #undef THIS_TYPE
        #undef NODE_SUBTYPE




//
// END MQL data structures                                                                                                                                */
//*********************************************************************************************************************************************************/
#endif // LIB_MQLPLUS_BASE_OBJECT_BINARY_TREE_NODE_MQH_INCLUDED
