#ifndef LIB_MQLPLUS_BASE_OBJECT_RB_TREE_NODE_MQH_INCLUDED
#define LIB_MQLPLUS_BASE_OBJECT_RB_TREE_NODE_MQH_INCLUDED
#property version "1.0";
/**********************************************************************************
 * Copyright (C) 2010-2022 Dominik Egert <info@freie-netze.de>
 *
 * This file is the objects include file.
 *
 * MQLplus, including this file may not be copied and/or distributed
 * without explecit permit by the author.
 * Author Dominik Egert / Freie Netze UG.
 **********************************************************************************
 *
 *  Version: 1.0
 *  State: production
 *
 *  File information
 *  ================
 *
 *
 *
*/
#ifdef DBG_MSG_TRACE_FILE_LOADER
    DBG_MSG_TRACE_FILE_LOADER;

#endif



// Incluse base node object
#include "lib_base_obj_node_tree_binary.mqh"





/*********************************************************************************************************************************************************/
/*                                                                                                                                                       */
/* MQLplus data structures                                                                                                                               */
/*                                                                                                                                                       */
/*********************************************************************************************************************************************************/


    ///////////////////////////////////////
    //
    //  Object Red Black Tree Node
    //

    // Base Red Black Tree object

        #define THIS_TYPE       _object_rb_t_node<USER_DATA>
        #define NODE_SUBTYPE    _object_binary_t_node<USER_DATA>

        template <typename USER_DATA>
        class _object_rb_t_node : public NODE_SUBTYPE
        {
            protected:
            // Local storage

                uchar		_state;                         // State 1 = Red, 0 = Black


            public:
            // Constructors / Destructor

                // Default constructor
                _object_rb_t_node() :
                    NODE_SUBTYPE    (),
                    _state          (1)
                { };

                // Copy constructor
                _object_rb_t_node(const THIS_TYPE& p_in)
                {
                    remove((THIS_TYPE*)_parent);
                    NODE_SUBTYPE(p_in);
                    _state = p_in._state;
                };

                // Destructor
                ~_object_rb_t_node()
                { remove((THIS_TYPE*)_parent); };


            // Tree element functions

                // Insert node into tree
                THIS_TYPE* insert(THIS_TYPE* _root)
                {
                    // Insert node
                    _root                                   = NODE_SUBTYPE::insert(_root, true);

                    // Validate insert operation
                    if( (_root == NULL)
                     && (_parent == NULL) )
                    { return(NULL); }

                    // Update root node
                    _root._state                            = NULL;

                    // Return
                    return((_parent._parent != NULL) ? _insert_fixup(_root) : _root);
                };

                // Remove node from tree
                THIS_TYPE* remove(THIS_TYPE* _root)
                {
                    // Validate node
                    if(_root == NULL)
                    { return(NULL); }

                    // Local init
                    const uchar this_state                  = _state;
                    bool        red_leaf                    = (this_state != NULL) && (_left == NULL) && (_right == NULL);
                    bool        right_double_red            = false;
                    bool        left_double_red             = false;
                    bool        b_fixup                     = false;
//DBG_BREAK_CONDITION_ON_ID(rb_tree);

                    // Remove
                    THIS_TYPE*  tmp_ptr 		            = NODE_SUBTYPE::remove(_root, true);
                    _root                                   = (tmp_ptr != NULL) ? tmp_ptr : _root;
                    tmp_ptr                                 = ::GetPointer(this);

                    // Check if last node
                    if( (_root == tmp_ptr)
                       && (_right == NULL)
                       && (_left == NULL) )
                    { return(NULL); }

                    // Swap states, check fixup requirement
                    if(_left != NULL)
                    {
                        _state                              = ((THIS_TYPE*)_left)._state;
                        b_fixup                             = (_right == NULL) && (_state == NULL);
                        right_double_red                    = (this_state != NULL) && (_left._right != NULL) && (((THIS_TYPE*)_left._right)._state != NULL);
                        left_double_red                     = (this_state != NULL) && (_left._left != NULL)  && (((THIS_TYPE*)_left._left)._state != NULL);
                        red_leaf                            = (_right == NULL) && (_state != NULL);
                        ((THIS_TYPE*)_left)._state          = this_state;
                        b_fixup                            |= (((_state + this_state) != 1) && (!red_leaf)) || right_double_red || left_double_red;
                    }
                    else
                    { 
                        _state                              = NULL; 
                        b_fixup                             = (!red_leaf) && ((_state + this_state) != 1);
                    }

                    // Perform fixup
                    //  Fixup is not required if
                    //  - either removed node or replacement node is red.
                    //  - removed and replaced node are red and replacement-node is leaf-node.
                    if( (b_fixup)
                     && (_root != tmp_ptr) )
                    {
                        // Fixing is done on the reconnected child node from the removed node.
                        // This node is a child of the sub_nodes parent. - Stored in _parent
                        //
                        // To figure if the fixup node is left or right from the sub_parent
                        //
                        // (_left holds successor node)
                        // (_right holds successors right child node)
                        // (_parent holds parent of node to be deleted ??)
                        //
                        // Replacement node is a NIL-Node
                        // We need a placeholder
                        // Temporary reconnect this node as NULL-Node
                        //  and perform fixup as required,
                        //  remove after fixup from tree again.
                        if(_parent._left == NULL)
                        {
                            _parent._left                   = tmp_ptr;
                            tmp_ptr._parent                 = _parent;
                            tmp_ptr._left                   = NULL;
                            tmp_ptr._right                  = NULL;
                            _root                           = _remove_fixup(_root);
                            _parent._left                   = NULL; 
                        }

                        else if(_parent._right == NULL)
                        {
                            tmp_ptr._parent                 = (_left != NULL) ? _left : _parent;
                            _parent._right                  = tmp_ptr;
                            tmp_ptr._left                   = NULL;
                            tmp_ptr._right                  = NULL;
                            _root                           = _remove_fixup(_root);
                            _parent._right                  = NULL;
                        }

                        else if( (_parent._right == _right)
                              || (_left._right == _right)
                              || (right_double_red) )
                        { _root                             = ((THIS_TYPE*)_right)._remove_fixup(_root); }

                        else
                        { _root                             = ((THIS_TYPE*)_parent._left)._remove_fixup(_root); }
                    }

                    // Perform recoloring
                    else if( (_right != NULL)
                          && ((_state + this_state) == 1)
                          && ((_state + ((THIS_TYPE*)_right)._state) == 1) )
                    {
                        ((THIS_TYPE*)_right)._state         = NULL;
                    }
                    
                    // Update root nodes state
                    _root._state                            = NULL;

                    // Clear node
                    _left                                   = NULL;
                    _right                                  = NULL;
                    _parent                                 = NULL;

                    //Return
                    return(_root);
                };


            // Operators

                // Assignment operator
                void operator=(const THIS_TYPE& p_in)
                {
                    remove((THIS_TYPE*)_parent);
                    NODE_SUBTYPE::operator=((NODE_SUBTYPE)p_in);
                    _state = p_in._state;
                };

                // Assignment operator
                void operator=(THIS_TYPE* p_in)
                {
                    //remove(_parent);
                    NODE_SUBTYPE::operator=((NODE_SUBTYPE*)p_in);
                    _state = p_in._state;
                };


            private:
            // Red Black tree algorithm functions

                // Insert Fixup
                THIS_TYPE* _insert_fixup(THIS_TYPE* _root)
                {
                    THIS_TYPE* new_node = ::GetPointer(this);
                    THIS_TYPE* uncle    = NULL;
                    
                    // Traverse tree
                    while( (new_node != _root)
                        && (((THIS_TYPE*)new_node._parent)._state != NULL)
                        && (!_StopFlag) )
                    {
                        if(new_node._parent == new_node._parent._parent._left)
                        {
                            uncle                                                           = new_node._parent._parent._right;
                            if( (uncle != NULL)
                             && (uncle._state != NULL) )
                            {
                                ((THIS_TYPE*)new_node._parent)._state                       = NULL;
                                uncle._state                                                = NULL;
                                ((THIS_TYPE*)new_node._parent._parent)._state               = 1;
                                
                                new_node                                                    = new_node._parent._parent;
                            }
                            else
                            {
                                if(new_node == new_node._parent._right)
                                {
                                    new_node                                                = new_node._parent;
                                    _root                                                   = _left_rotate(new_node, _root);
                                }
                                ((THIS_TYPE*)new_node._parent)._state                       = NULL;
                                ((THIS_TYPE*)new_node._parent._parent)._state               = 1;
                                _root                                                       = _right_rotate(new_node._parent._parent, _root);
                            }
                        }
                        else
                        {
                            uncle                                                           = new_node._parent._parent._left;
                            if( (uncle != NULL)
                             && (uncle._state != NULL) )
                            {
                                ((THIS_TYPE*)new_node._parent)._state                       = NULL;
                                uncle._state                                                = NULL;
                                ((THIS_TYPE*)new_node._parent._parent)._state               = 1;
                                new_node                                                    = new_node._parent._parent;
                            }
                            else
                            {
                                if(new_node == new_node._parent._left)
                                {
                                    new_node                                                = new_node._parent;
                                    _root                                                   = _right_rotate(new_node, _root);
                                }
                                ((THIS_TYPE*)new_node._parent)._state                       = NULL;
                                ((THIS_TYPE*)new_node._parent._parent)._state               = 1;
                                _root                                                       = _left_rotate(new_node._parent._parent, _root);
                            }
                        }
                    }

                    // Clear root state
                    _root._state                                                            = NULL;

                    // Return
                    return(_root);
                };

                // Delete Fixup
                THIS_TYPE* _remove_fixup(THIS_TYPE* _root)
                {
                    // Local init
                    THIS_TYPE* sibling      = NULL;
                    THIS_TYPE* fixup_node   = ::GetPointer(this);

                    // Traverse tree
                    while( (fixup_node != _root)
                        && (fixup_node._state == NULL)
                        && (!_StopFlag) )
                    {
                        if(fixup_node == fixup_node._parent._left)
                        {
                            sibling                         								= fixup_node._parent._right;
                            if( (sibling != NULL)								
                             && (sibling._state != NULL) )								
                            {								
                                ((THIS_TYPE*)fixup_node._parent)._state     				= 1;
                                sibling._state              								= NULL;
                                _root                       								= _left_rotate(fixup_node._parent, _root);
                                sibling                     								= fixup_node._parent._right;
                            }
                            
                            if(sibling == NULL)
                            { return(_root); }

                            if( ( (sibling._left == NULL) 
							   || (((THIS_TYPE*)sibling._left)._state == NULL))
                             && ( (sibling._right == NULL) 
							   || (((THIS_TYPE*)sibling._right)._state == NULL)) )
                            {
                                sibling._state              								= 1;
                                fixup_node                    								= fixup_node._parent;
                            }
                            else
                            {
                                if( (sibling._right == NULL) 
								 || (((THIS_TYPE*)sibling._right)._state == NULL) )
                                {
                                    if(sibling._left != NULL)
                                    { ((THIS_TYPE*)sibling._left)._state  					= NULL; }
                                    sibling._state          								= 1;
                                    _root                   								= _right_rotate(sibling, _root);
                                    sibling                 								= fixup_node._parent._right;
                                }								
																
                                sibling._state              								= (fixup_node._parent != NULL) ? ((THIS_TYPE*)fixup_node._parent)._state : NULL;
                                if(fixup_node._parent != NULL)								
                                { ((THIS_TYPE*)fixup_node._parent)._state   				= NULL; }
                                if(sibling._right != NULL)								
                                { ((THIS_TYPE*)sibling._right)._state     					= NULL; }
                                _root                       								= _left_rotate(fixup_node._parent, _root);
                                fixup_node                    								= _root;
                            }
                        }
                        else
                        {
                            sibling                         								= fixup_node._parent._left;
                            if( (sibling != NULL) 								
                             && (sibling._state != NULL) )								
                            {
                                ((THIS_TYPE*)fixup_node._parent)._state     				= 1;
                                sibling._state              								= NULL;
                                _root                       								= _right_rotate(fixup_node._parent, _root);
                                sibling                     								= fixup_node._parent._left;
                            }
                            
                            if(sibling == NULL)
                            { return(_root); }

                            if( ( (sibling._right == NULL) 
							   || (((THIS_TYPE*)sibling._right)._state == NULL))
                             && ( (sibling._left == NULL) 
							   || (((THIS_TYPE*)sibling._left)._state == NULL)) )
                            {
                                sibling._state              								= 1;
                                fixup_node                    								= fixup_node._parent;
                            }
                            else
                            {
                                if( (sibling._left == NULL) 
								 || (((THIS_TYPE*)sibling._left)._state == NULL) )
                                {
                                    if(sibling._right != NULL)
                                    { ((THIS_TYPE*)sibling._right)._state 					= NULL; }
                                    sibling._state          								= 1;
                                    _root                   								= _left_rotate(sibling, _root);
                                    sibling                 								= fixup_node._parent._left;
                                }								
                                sibling._state              								= (fixup_node._parent != NULL) ? ((THIS_TYPE*)fixup_node._parent)._state : NULL;
                                if(fixup_node._parent != NULL)								
                                { ((THIS_TYPE*)fixup_node._parent)._state   				= NULL; }
                                if(sibling._left != NULL)								
                                { ((THIS_TYPE*)sibling._left)._state      					= NULL; }
                                _root                       								= _right_rotate(fixup_node._parent, _root);
                                fixup_node                    								= _root;
                            }								
                        }								
                    }								
								
                    // Cleanup								
                    fixup_node._state                         								= NULL;
                    _root._parent                           								= NULL;

                    // Return
                    return(_root);
                };

                // Left rotation
                THIS_TYPE* _left_rotate(THIS_TYPE* x, THIS_TYPE* _root)
                {
                    // Local init
                    THIS_TYPE* y                            = x._right;

                    // Left rotate
                    x._right                                = (y != NULL) ? y._left : NULL;

                    if(y._left != NULL)
                    { y._left._parent                       = x; }
                    y._parent                               = x._parent;

                    if(x._parent == NULL)
                    { _root                                 = y; }
                    else if(x == x._parent._left)
                    { x._parent._left                       = y; }
                    else
                    { x._parent._right                      = y; }
                    y._left                                 = x;
                    x._parent                               = y;

                    // Cleanup
                    _root._parent                           = NULL;

                    // Return
                    return(_root);
                };

                // Right rotate
                THIS_TYPE* _right_rotate(THIS_TYPE* x, THIS_TYPE* _root)
                {
                    // Local init
                    THIS_TYPE* y                            = x._left;

                    // Right rotate
                    x._left                                 = (y != NULL) ? y._right : NULL;

                    if(y._right != NULL)
                    { y._right._parent                      = x; }
                    y._parent                               = x._parent;

                    if(x._parent == NULL)
                    { _root                                 = y; }
                    else if(x == x._parent._right)
                    { x._parent._right                      = y; }
                    else
                    { x._parent._left                       = y; }
                    y._right                                = x;
                    x._parent                               = y;

                    // Cleanup
                    _root._parent                           = NULL;

                    // Return
                    return(_root);
                };


            public:
            // Debugging helper

                const string _node_to_string(int& count)
                { count++; return(StringFormat("Parent Value: %s; Color: %s; Value: %s", (_parent != NULL) ? __obj_to_string(_parent.this_obj) : "Root-Node", (_state == NULL) ? "Black" : "Red", __obj_to_string(this_obj))); };
        };

        #undef THIS_TYPE
        #undef NODE_SUBTYPE




//
// END MQL data structures                                                                                                                                */
//*********************************************************************************************************************************************************/
#endif // LIB_MQLPLUS_BASE_OBJECT_RB_TREE_NODE_MQH_INCLUDED
