#ifndef LIB_MQLPLUS_BASE_OBJECT_TYPECAST_MQH_INCLUDED
#define LIB_MQLPLUS_BASE_OBJECT_TYPECAST_MQH_INCLUDED
#property version "1.0";
/**********************************************************************************
 * Copyright (C) 2010-2022 Dominik Egert <info@freie-netze.de>
 *
 * This file is the objects include file.
 *
 * MQLplus, including this file may not be copied and/or distributed
 * without explecit permit by the author.
 * Author Dominik Egert / Freie Netze UG.
 **********************************************************************************
 *
 *  Version: 1.0
 *  State: production
 *
 *  File information
 *  ================
 *
 *
 *
*/
#ifdef DBG_MSG_TRACE_FILE_LOADER
    DBG_MSG_TRACE_FILE_LOADER;

#endif



/*********************************************************************************************************************************************************/
/*                                                                                                                                                       */
/* MQLplus data structures                                                                                                                               */
/*                                                                                                                                                       */
/*********************************************************************************************************************************************************/


    ///////////////////////////////////////
    //
    //  Object Node
    //

    // Type cast helper class


        template <typename USER_DATA, typename NODE_SUBTYPE, typename NODE_TYPE>
        class _object_typecast_node : protected NODE_SUBTYPE
        {
            public:
            // Default constructor

                _object_typecast_node() :
                    NODE_SUBTYPE()
                { };

                _object_typecast_node(const NODE_SUBTYPE& p_in) :
                    NODE_SUBTYPE(p_in)
                { };


            public:
            // Object auto create function

                template <typename CUSTOM_DATA>
                const bool _autocreate(const CUSTOM_DATA*& p_assign)
                { return(__autocreate(this_obj, p_assign)); };
  
                template <typename CUSTOM_DATA>
                const bool _autocreate(CUSTOM_DATA* p_assign)
                { return(__autocreate(this_obj, p_assign)); };
 
                template <typename CUSTOM_DATA>
                const bool _autocreate(CUSTOM_DATA& p_assign)
                { return(__autocreate(this_obj, p_assign)); };


            private:
            // Internal auto-create selector

                template <typename TARGET_TYPE, typename CUSTOM_DATA>
                const bool __autocreate(TARGET_TYPE*& p_this, CUSTOM_DATA& p_assign)
                {
                    p_this = (dynamic_cast<const TARGET_TYPE*>(&p_assign) == NULL) ? new TARGET_TYPE(p_assign) : new CUSTOM_DATA(p_assign); 
                    return(p_this != NULL);
                };

                template <typename TARGET_TYPE, typename CUSTOM_DATA>
                const bool __autocreate(TARGET_TYPE& p_this, CUSTOM_DATA* p_assign)
                {
                    p_this = p_assign; 
                    return(true); 
                };
 
                template <typename TARGET_TYPE, typename CUSTOM_DATA>
                const bool __autocreate(TARGET_TYPE*& p_this, const CUSTOM_DATA*& p_assign)
                {
                    p_this = (dynamic_cast<const TARGET_TYPE*>(p_assign) == NULL) ? new TARGET_TYPE(p_assign) : new CUSTOM_DATA(p_assign); 
                    return(p_this != NULL);
                };
  
                template <typename TARGET_TYPE, typename CUSTOM_DATA>
                const bool __autocreate(TARGET_TYPE*& p_this, CUSTOM_DATA*& p_assign)
                {
                    p_this = (dynamic_cast<TARGET_TYPE*>(p_assign) == NULL) ? new TARGET_TYPE(p_assign) : new CUSTOM_DATA(p_assign); 
                    return(p_this != NULL);
                };
  
                template <typename TARGET_TYPE, typename CUSTOM_DATA>
                const bool __autocreate(TARGET_TYPE& p_this, CUSTOM_DATA& p_assign)
                {
                    p_this = p_assign; 
                    return(true); 
                };


            public:
            // Object auto delete function

                template <typename CUSTOM_DATA>
                void _autodelete(CUSTOM_DATA& p_assign) const
                { __autodelete(this_obj, p_assign); };

                void _autodelete() const 
                { __autodelete(this_obj, this_obj); }


            private:
            // Internal delete selector

                template <typename TARGET_TYPE, typename CUSTOM_DATA>
                void __autodelete(TARGET_TYPE*& p_this, CUSTOM_DATA& p_assign) const
                { delete(this_obj); };

                template <typename TARGET_TYPE, typename CUSTOM_DATA>
                void __autodelete(TARGET_TYPE*& p_this, const CUSTOM_DATA*& p_assign) const
                { delete(this_obj); };

                template <typename TARGET_TYPE, typename CUSTOM_DATA>
                void __autodelete(TARGET_TYPE*& p_this, CUSTOM_DATA* p_assign) const
                { delete(this_obj); };

                template <typename TARGET_TYPE>
                void __autodelete(TARGET_TYPE*& p_this, USER_DATA& p_assign) const
                { };

                template <typename TARGET_TYPE, typename CUSTOM_DATA>
                void __autodelete(TARGET_TYPE& p_this, CUSTOM_DATA* p_assign) const
                { };
                
                template <typename TARGET_TYPE, typename CUSTOM_DATA>
                void __autodelete(TARGET_TYPE& p_this, CUSTOM_DATA& p_assign) const
                { };


            public:
            // Object cast compare

                // Greater            
                template <typename CALL_TYPE>
                const bool _cast_compare_operator_greater(CALL_TYPE* p_in) const
                { return(__cast_compare_operator_greater(this_obj, p_in)); };
            
                template <typename CALL_TYPE>
                const bool _cast_compare_operator_greater(CALL_TYPE& p_in) const
                { return(__cast_compare_operator_greater(this_obj, p_in)); };
            
                template <typename CALL_TYPE>
                const bool _cast_compare_operator_greater(CALL_TYPE p_in) const
                { return(__cast_compare_operator_greater(this_obj, p_in)); };

                // Smaller
                template <typename CALL_TYPE>
                const bool _cast_compare_operator_smaller(CALL_TYPE* p_in) const
                { return(__cast_compare_operator_smaller(this_obj, p_in)); };
            
                template <typename CALL_TYPE>
                const bool _cast_compare_operator_smaller(CALL_TYPE& p_in) const
                { return(__cast_compare_operator_smaller(this_obj, p_in)); };
            
                template <typename CALL_TYPE>
                const bool _cast_compare_operator_smaller(CALL_TYPE p_in) const
                { return(__cast_compare_operator_smaller(this_obj, p_in)); };

            
            private:
            // Internal compare function

                // Greater
                template <typename TARGET_TYPE, typename SOURCE_TYPE>
                const bool __cast_compare_operator_greater(TARGET_TYPE*& p_out, SOURCE_TYPE* p_in) const
                { 
                    const TARGET_TYPE* p_tmp = dynamic_cast<const TARGET_TYPE*>(p_in);
                    return((p_tmp == NULL) ? (p_out > TARGET_TYPE(p_in)) : (p_out > p_tmp));
                };

                template <typename TARGET_TYPE, typename SOURCE_TYPE>
                const bool __cast_compare_operator_greater(TARGET_TYPE& p_out, SOURCE_TYPE* p_in) const
                { 
                    const TARGET_TYPE* p_tmp = dynamic_cast<const TARGET_TYPE*>(p_in);
                    return((p_tmp == NULL) ? (p_out > TARGET_TYPE(p_in)) : (p_out > p_tmp));
                };

                template <typename TARGET_TYPE, typename SOURCE_TYPE>
                const bool __cast_compare_operator_greater(TARGET_TYPE*& p_out, SOURCE_TYPE& p_in) const
                { 
                    const TARGET_TYPE* p_tmp = dynamic_cast<const TARGET_TYPE*>(&p_in);
                    return((p_tmp == NULL) ? (p_out > TARGET_TYPE(p_in)) : (p_out > p_tmp));
                };

                template <typename TARGET_TYPE, typename SOURCE_TYPE>
                const bool __cast_compare_operator_greater(TARGET_TYPE& p_out, SOURCE_TYPE& p_in) const
                { return(p_out > TARGET_TYPE(p_in)); };

                // Smaller
                template <typename TARGET_TYPE, typename SOURCE_TYPE>
                const bool __cast_compare_operator_smaller(TARGET_TYPE*& p_out, SOURCE_TYPE* p_in) const
                { 
                    const TARGET_TYPE* p_tmp = dynamic_cast<const TARGET_TYPE*>(p_in);
                    return((p_tmp == NULL) ? (TARGET_TYPE(p_in) > p_out) : (p_tmp > p_out));
                };

                template <typename TARGET_TYPE, typename SOURCE_TYPE>
                const bool __cast_compare_operator_smaller(TARGET_TYPE& p_out, SOURCE_TYPE* p_in) const
                { 
                    const TARGET_TYPE* p_tmp = dynamic_cast<const TARGET_TYPE*>(p_in);
                    return((p_tmp == NULL) ? (TARGET_TYPE(p_in) > p_out) : (p_tmp > p_out));
                };

                template <typename TARGET_TYPE, typename SOURCE_TYPE>
                const bool __cast_compare_operator_smaller(TARGET_TYPE*& p_out, SOURCE_TYPE& p_in) const
                { 
                    const TARGET_TYPE* p_tmp = dynamic_cast<const TARGET_TYPE*>(&p_in);
                    return((p_tmp == NULL) ? (TARGET_TYPE(p_in) > p_out) : (p_tmp > p_out));
                };

                template <typename TARGET_TYPE, typename SOURCE_TYPE>
                const bool __cast_compare_operator_smaller(TARGET_TYPE& p_out, SOURCE_TYPE& p_in) const
                { return(TARGET_TYPE(p_in) > p_out); };
        };








        template <typename USER_DATA, typename NODE_SUBTYPE, typename NODE_TYPE>
        class _object_typecast_tree_node : public _object_typecast_node<USER_DATA, NODE_SUBTYPE, NODE_TYPE>
        {
            public:
            // Default constructor

                _object_typecast_tree_node() :
                    _object_typecast_node()
                { };

                _object_typecast_tree_node(const NODE_SUBTYPE& p_in) :
                    _object_typecast_node(p_in)
                { };


            public:
            // Universal greater operator

                template <typename CALL_TYPE>
                NODE_TYPE* _cast_next_greater_operator(CALL_TYPE* p_in, const bool or_equal)
                { return(__cast_next_greater_operator(this_obj, p_in, or_equal)); };
            
                template <typename CALL_TYPE>
                NODE_TYPE* _cast_next_greater_operator(CALL_TYPE& p_in, const bool or_equal)
                { return(__cast_next_greater_operator(this_obj, p_in, or_equal)); };
            
                template <typename CALL_TYPE>
                NODE_TYPE* _cast_next_greater_operator(CALL_TYPE p_in, const bool or_equal) 
                { return(__cast_next_greater_operator(this_obj, p_in, or_equal)); };

                NODE_TYPE* _cast_next_greater_operator(const USER_DATA& p_in, const bool or_equal) 
                { return(__cast_next_greater_operator(this_obj, p_in, or_equal)); };

                NODE_TYPE* _cast_next_greater_operator(USER_DATA& p_in, const bool or_equal) 
                { return(__cast_next_greater_operator(this_obj, p_in, or_equal)); };


            private:
            // Normalized input indexing operator

                NODE_TYPE* __cast_next_greater_operator(USER_DATA& p_out, const USER_DATA& p_in, const bool or_equal) 
                { return(NODE_SUBTYPE::greater(p_in, or_equal)); };

                NODE_TYPE* __cast_next_greater_operator(USER_DATA& p_out, USER_DATA& p_in, const bool or_equal) 
                { return(NODE_SUBTYPE::greater(p_in, or_equal)); };

                template <typename TARGET_TYPE, typename SOURCE_TYPE>
                NODE_TYPE* __cast_next_greater_operator(TARGET_TYPE*& p_out, SOURCE_TYPE* p_in, const bool or_equal)
                { 
                    const TARGET_TYPE* p_tmp = dynamic_cast<const TARGET_TYPE*>(p_in);
                    return((p_tmp == NULL) ? NODE_SUBTYPE::greater(TARGET_TYPE(p_in), or_equal) : NODE_SUBTYPE::greater(p_tmp, or_equal));
                };

                template <typename TARGET_TYPE, typename SOURCE_TYPE>
                NODE_TYPE* __cast_next_greater_operator(TARGET_TYPE& p_out, SOURCE_TYPE* p_in, const bool or_equal)
                { 
                    const TARGET_TYPE* p_tmp = dynamic_cast<const TARGET_TYPE*>(p_in);
                    return((p_tmp == NULL) ? NODE_SUBTYPE::greater(TARGET_TYPE(p_in), or_equal) : NODE_SUBTYPE::greater(p_tmp, or_equal));
                };

                template <typename TARGET_TYPE, typename SOURCE_TYPE>
                NODE_TYPE* __cast_next_greater_operator(TARGET_TYPE*& p_out, SOURCE_TYPE& p_in, const bool or_equal)
                { 
                    const TARGET_TYPE* p_tmp = dynamic_cast<const TARGET_TYPE*>(&p_in);
                    return((p_tmp == NULL) ? NODE_SUBTYPE::greater(TARGET_TYPE(p_in), or_equal) : NODE_SUBTYPE::greater(p_tmp, or_equal));
                };

                template <typename TARGET_TYPE, typename SOURCE_TYPE>
                NODE_TYPE* __cast_next_greater_operator(TARGET_TYPE& p_out, SOURCE_TYPE& p_in, const bool or_equal)
                { return(NODE_SUBTYPE::greater(TARGET_TYPE(p_in) , or_equal)); };

                template <typename TARGET_TYPE, typename SOURCE_TYPE>
                NODE_TYPE* __cast_next_greater_operator(TARGET_TYPE& p_out, SOURCE_TYPE p_in, const bool or_equal)
                { return(NODE_SUBTYPE::greater(p_in, or_equal)); };


            public:
            // Universal next_smaller operator

                template <typename CALL_TYPE>
                NODE_TYPE* _cast_next_smaller_operator(CALL_TYPE* p_in, const bool or_equal)
                { return(__cast_next_smaller_operator(this_obj, p_in, or_equal)); };
            
                template <typename CALL_TYPE>
                NODE_TYPE* _cast_next_smaller_operator(CALL_TYPE& p_in, const bool or_equal)
                { return(__cast_next_smaller_operator(this_obj, p_in, or_equal)); };
            
                template <typename CALL_TYPE>
                NODE_TYPE* _cast_next_smaller_operator(CALL_TYPE p_in, const bool or_equal) 
                { return(__cast_next_smaller_operator(this_obj, p_in, or_equal)); };

                NODE_TYPE* _cast_next_smaller_operator(const USER_DATA& p_in, const bool or_equal) 
                { return(__cast_next_smaller_operator(this_obj, p_in, or_equal)); };

                NODE_TYPE* _cast_next_smaller_operator(USER_DATA& p_in, const bool or_equal) 
                { return(__cast_next_smaller_operator(this_obj, p_in, or_equal)); };


            private:
            // Normalized input indexing operator

                NODE_TYPE* __cast_next_smaller_operator(USER_DATA& p_out, const USER_DATA& p_in, const bool or_equal) 
                { return(NODE_SUBTYPE::smaller(p_in, or_equal)); };

                NODE_TYPE* __cast_next_smaller_operator(USER_DATA& p_out, USER_DATA& p_in, const bool or_equal) 
                { return(NODE_SUBTYPE::smaller(p_in, or_equal)); };

                template <typename TARGET_TYPE, typename SOURCE_TYPE>
                NODE_TYPE* __cast_next_smaller_operator(TARGET_TYPE*& p_out, SOURCE_TYPE* p_in, const bool or_equal)
                { 
                    const TARGET_TYPE* p_tmp = dynamic_cast<const TARGET_TYPE*>(p_in);
                    return((p_tmp == NULL) ? NODE_SUBTYPE::smaller(TARGET_TYPE(p_in), or_equal) : NODE_SUBTYPE::smaller(p_tmp, or_equal));
                };

                template <typename TARGET_TYPE, typename SOURCE_TYPE>
                NODE_TYPE* __cast_next_smaller_operator(TARGET_TYPE& p_out, SOURCE_TYPE* p_in, const bool or_equal)
                { 
                    const TARGET_TYPE* p_tmp = dynamic_cast<const TARGET_TYPE*>(p_in);
                    return((p_tmp == NULL) ? NODE_SUBTYPE::smaller(TARGET_TYPE(p_in), or_equal) : NODE_SUBTYPE::smaller(p_tmp, or_equal));
                };

                template <typename TARGET_TYPE, typename SOURCE_TYPE>
                NODE_TYPE* __cast_next_smaller_operator(TARGET_TYPE*& p_out, SOURCE_TYPE& p_in, const bool or_equal)
                { 
                    const TARGET_TYPE* p_tmp = dynamic_cast<const TARGET_TYPE*>(&p_in);
                    return((p_tmp == NULL) ? NODE_SUBTYPE::smaller(TARGET_TYPE(p_in), or_equal) : NODE_SUBTYPE::smaller(p_tmp, or_equal));
                };

                template <typename TARGET_TYPE, typename SOURCE_TYPE>
                NODE_TYPE* __cast_next_smaller_operator(TARGET_TYPE& p_out, SOURCE_TYPE& p_in, const bool or_equal)
                { return(NODE_SUBTYPE::smaller(TARGET_TYPE(p_in), or_equal)); };

                template <typename TARGET_TYPE, typename SOURCE_TYPE>
                NODE_TYPE* __cast_next_smaller_operator(TARGET_TYPE& p_out, SOURCE_TYPE p_in, const bool or_equal)
                { return(NODE_SUBTYPE::smaller(p_in, or_equal)); };


            public:
            // Universal indexing operator

                template <typename CALL_TYPE>
                NODE_TYPE* _cast_indexing_operator(CALL_TYPE* p_in)
                { return(__cast_indexing_operator(this_obj, p_in)); };
            
                template <typename CALL_TYPE>
                NODE_TYPE* _cast_indexing_operator(CALL_TYPE& p_in)
                { return(__cast_indexing_operator(this_obj, p_in)); };
            
                template <typename CALL_TYPE>
                NODE_TYPE* _cast_indexing_operator(CALL_TYPE p_in) 
                { return(__cast_indexing_operator(this_obj, p_in)); };

                NODE_TYPE* _cast_indexing_operator(const USER_DATA& p_in) 
                { return(__cast_indexing_operator(this_obj, p_in)); };

                NODE_TYPE* _cast_indexing_operator(USER_DATA& p_in) 
                { return(__cast_indexing_operator(this_obj, p_in)); };


            private:
            // Normalized input indexing operator

                NODE_TYPE* __cast_indexing_operator(USER_DATA& p_out, const USER_DATA& p_in) 
                { return(NODE_SUBTYPE::operator[](p_in)); };

                NODE_TYPE* __cast_indexing_operator(USER_DATA& p_out, USER_DATA& p_in) 
                { return(NODE_SUBTYPE::operator[](p_in)); };

                template <typename TARGET_TYPE, typename SOURCE_TYPE>
                NODE_TYPE* __cast_indexing_operator(TARGET_TYPE*& p_out, SOURCE_TYPE* p_in)
                { 
                    const TARGET_TYPE* p_tmp = dynamic_cast<const TARGET_TYPE*>(p_in);
                    return((p_tmp == NULL) ? NODE_SUBTYPE::operator[](TARGET_TYPE(p_in)) : NODE_SUBTYPE::operator[](p_tmp));
                };

                template <typename TARGET_TYPE, typename SOURCE_TYPE>
                NODE_TYPE* __cast_indexing_operator(TARGET_TYPE& p_out, SOURCE_TYPE* p_in)
                { 
                    const TARGET_TYPE* p_tmp = dynamic_cast<const TARGET_TYPE*>(p_in);
                    return((p_tmp == NULL) ? NODE_SUBTYPE::operator[](TARGET_TYPE(p_in)) : NODE_SUBTYPE::operator[](p_tmp));
                };

                template <typename TARGET_TYPE, typename SOURCE_TYPE>
                NODE_TYPE* __cast_indexing_operator(TARGET_TYPE*& p_out, SOURCE_TYPE& p_in)
                { 
                    const TARGET_TYPE* p_tmp = dynamic_cast<const TARGET_TYPE*>(&p_in);
                    return((p_tmp == NULL) ? NODE_SUBTYPE::operator[](TARGET_TYPE(p_in)) : NODE_SUBTYPE::operator[](p_tmp));
                };

                template <typename TARGET_TYPE, typename SOURCE_TYPE>
                NODE_TYPE* __cast_indexing_operator(TARGET_TYPE& p_out, SOURCE_TYPE& p_in)
                { return(NODE_SUBTYPE::operator[](TARGET_TYPE(p_in))); };

                template <typename TARGET_TYPE, typename SOURCE_TYPE>
                NODE_TYPE* __cast_indexing_operator(TARGET_TYPE& p_out, SOURCE_TYPE p_in)
                { return(NODE_SUBTYPE::operator[](p_in)); };

        };














//
// END MQL data structures                                                                                                                                */
//*********************************************************************************************************************************************************/
#endif // LIB_MQLPLUS_BASE_OBJECT_TYPECAST_MQH_INCLUDED
