#ifndef LIB_MQLPLUS_OBJECT_LIST_NODE_MQH_INCLUDED
#define LIB_MQLPLUS_OBJECT_LIST_NODE_MQH_INCLUDED
#property version "1.0";
/**********************************************************************************
 * Copyright (C) 2010-2022 Dominik Egert <info@freie-netze.de>
 *
 * This file is the objects include file.
 *
 * MQLplus, including this file may not be copied and/or distributed
 * without explecit permit by the author.
 * Author Dominik Egert / Freie Netze UG.
 **********************************************************************************
 *
 *  Version: 1.0
 *  State: production
 *
 *  File information
 *  ================
 *
 *
 *
*/
#ifdef DBG_MSG_TRACE_FILE_LOADER
    DBG_MSG_TRACE_FILE_LOADER;

#endif



// Incluse base object
#include "lib_base_obj_node_linkedlist.mqh"



/*********************************************************************************************************************************************************/
/*                                                                                                                                                       */
/* MQLplus data structures                                                                                                                               */
/*                                                                                                                                                       */
/*********************************************************************************************************************************************************/



    ///////////////////////////////////////
    //
    //  Object List Node
    //

    // Data holding object

        // Derived simple type class object
		#define NODE_SUBTYPE	_object_l_node<USER_DATA>
		#define NODE_TYPE		object_list_node<USER_DATA>

        template <typename USER_DATA>
        class object_list_node : protected NODE_SUBTYPE
        {
            public:
            // Constructors / Destructor

                // Default constructor
                object_list_node() : _object_l_node()                                       { };

                // Copy constructor
                object_list_node(const NODE_TYPE& p_in)                   					{ NODE_SUBTYPE(p_in); };

                // Value constructor
                object_list_node(USER_DATA& p_in)                                           { this_obj = p_in; };

                // Destructor
                ~object_list_node()                                                         { };


            // Element selectors

                // First chain element
                NODE_TYPE* first_element()                                					{ return((NODE_TYPE*)left()); };
					
                // Last chain element					
                NODE_TYPE* last_element()                                 					{ return((NODE_TYPE*)right()); };
					
                // Preious element					
                NODE_TYPE* prev_element()                                 					{ return((NODE_TYPE*)_left); };
					
                // Next element					
                NODE_TYPE* next_element()                                 					{ return((NODE_TYPE*)_right); };
					
					
            // List element functions					
					
                // Append element					
                void append(NODE_TYPE& p_in)                              					{ NODE_SUBTYPE::append(p_in); };
					
                // Prepend element					
                void prepend(NODE_TYPE& p_in)                             					{ NODE_SUBTYPE::prepend(p_in); };
					
                // Insert element					
                void insert(NODE_TYPE& p_in)                              					{ NODE_SUBTYPE::insert(p_in); };

                // Append element
                void append(USER_DATA& p_in)                                                { NODE_TYPE* new_ptr = new NODE_TYPE(p_in); NODE_SUBTYPE::append((NODE_SUBTYPE*)new_ptr); };

                // Prepend element
                void prepend(USER_DATA& p_in)                                               { NODE_TYPE* new_ptr = new NODE_TYPE(p_in); NODE_SUBTYPE::prepend((NODE_SUBTYPE*)new_ptr); };

                // Insert element
                void insert(USER_DATA& p_in)                                                { NODE_TYPE* new_ptr = new NODE_TYPE(p_in); NODE_SUBTYPE::insert((NODE_SUBTYPE*)new_ptr); };

                // Get key value
                USER_DATA get_key() const                                                   { return(this_obj); };

                // Get object
                USER_DATA get_obj() const                                                   { return(this_obj); };

                // Detach object
                USER_DATA detach_obj()                                                      { return(this_obj); };

                // Remove object from chain
                void remove(const bool do_not_delete = false)                               { NODE_SUBTYPE::remove(); };


            // Operators

                // Value assignment operator
                USER_DATA operator=(USER_DATA& p_in)                                        { this_obj = p_in; return(this_obj); };

                // Access operator
                NODE_TYPE* operator[](const int idx)                      					{ return((NODE_TYPE*)NODE_SUBTYPE::operator[](idx)); };

                // Copy assignment operator
                void operator=(const NODE_TYPE& p_in)                     					{ NODE_SUBTYPE::operator=((NODE_SUBTYPE)p_in); };
        };

		#undef NODE_TYPE
		#undef NODE_SUBTYPE



        // Derived complex type class object for pointers
		#define NODE_SUBTYPE	_object_l_node<USER_DATA*>
		#define NODE_TYPE		ptr_object_list_node<USER_DATA>

        template <typename USER_DATA>
        class ptr_object_list_node : protected NODE_SUBTYPE
        {
            public:
            // Constructors / Destructor

                // Default constructor
                ptr_object_list_node() : _object_l_node()                                   { };

                // Copy constructor
                ptr_object_list_node(const NODE_TYPE& p_in)           						{ NODE_SUBTYPE(p_in); }

                // Value constructor
                ptr_object_list_node(const USER_DATA& p_in)                                 { this_obj = new USER_DATA; this_obj = p_in; };
                ptr_object_list_node(USER_DATA* p_in)                                       { this_obj = p_in; };

                // Destructor
                ~ptr_object_list_node()                                                     { if((this_obj != NULL) && (CheckPointer(this_obj) == POINTER_DYNAMIC)) { delete(this_obj); } };


            // Element selectors

                // First chain element
                NODE_TYPE* first_element()                            						{ return((NODE_TYPE*)left()); };
						
                // Last chain element						
                NODE_TYPE* last_element()                             						{ return((NODE_TYPE*)right()); };
						
                // Preious element						
                NODE_TYPE* prev_element()                             						{ return((NODE_TYPE*)_left); };
						
                // Next element						
                NODE_TYPE* next_element()                             						{ return((NODE_TYPE*)_right); };
						
						
            // List element functions						
						
                // Append element						
                void append(NODE_TYPE& p_in)                          						{ NODE_SUBTYPE::append(p_in); };
						
                // Prepend element						
                void prepend(NODE_TYPE& p_in)                         						{ NODE_SUBTYPE::prepend(p_in); };
						
                // Insert element						
                void insert(NODE_TYPE& p_in)                          						{ NODE_SUBTYPE::insert(p_in); };

                // Append element
                void append(USER_DATA* p_in)                                                { NODE_TYPE* new_ptr = new NODE_TYPE(p_in); NODE_SUBTYPE::append((NODE_SUBTYPE*)new_ptr); };
                void append(USER_DATA& p_in)                                                { NODE_TYPE* new_ptr = new NODE_TYPE(new USER_DATA(p_in)); NODE_SUBTYPE::append((NODE_SUBTYPE*)new_ptr); };

                // Prepend element
                void prepend(USER_DATA* p_in)                                               { NODE_TYPE* new_ptr = new NODE_TYPE(p_in); NODE_SUBTYPE::prepend((NODE_SUBTYPE*)new_ptr); };
                void prepend(USER_DATA& p_in)                                               { NODE_TYPE* new_ptr = new NODE_TYPE(new USER_DATA(p_in)); NODE_SUBTYPE::prepend((NODE_SUBTYPE*)new_ptr); };

                // Insert element
                void insert(USER_DATA* p_in)                                                { NODE_TYPE* new_ptr = new NODE_TYPE(p_in); NODE_SUBTYPE::insert((NODE_SUBTYPE*)new_ptr); };
                void insert(USER_DATA& p_in)                                                { NODE_TYPE* new_ptr = new NODE_TYPE(new USER_DATA(p_in)); NODE_SUBTYPE::insert((NODE_SUBTYPE*)new_ptr); };

                // Object handlers
                USER_DATA* get_key() const                                                  { return(this_obj); };
                USER_DATA* get_obj() const                                                  { return(this_obj); };
                USER_DATA* detach_obj()                                                     { USER_DATA* tmp = this_obj; this_obj = NULL; return(tmp); }

                // Remove object from chain
                void remove(const bool do_not_delete = false)                               { if(do_not_delete) { detach_obj(); }; NODE_SUBTYPE::remove(); };


            // Operators

                // Value assignment operator
                USER_DATA* operator=(USER_DATA& p_in)                                       { this_obj = new USER_DATA(p_in); return(this_obj); };
                USER_DATA* operator=(USER_DATA* p_in)                                       { this_obj = p_in; return(this_obj); };

                // Access operator
                NODE_TYPE* operator[](const int idx)                  						{ return((NODE_TYPE*)NODE_SUBTYPE::operator[](idx)); };
						
                // Copy assignment operator						
                void operator=(const NODE_TYPE& p_in)                 						{ NODE_SUBTYPE::operator=((NODE_SUBTYPE)p_in); };
        };

		#undef NODE_TYPE
		#undef NODE_SUBTYPE




//
// END MQL data structures                                                                                                                                */
//*********************************************************************************************************************************************************/
#endif // LIB_MQLPLUS_OBJECT_LIST_NODE_MQH_INCLUDED
